% Roman G. Szafran and Mikita Dvykoza
% The influence of electrolyte flow hydrodynamics on the performance of a microfluidic dye-sensitized solar cell
% Applied Sciences, 2021

commandwindow
clc;
clear all;
close all;
format long;
% Fixed program parameters.
rho = 1000; % Fluid density [kg/m3].
mu = 1e-6; % Fluid kinematic density [m2/s].
nu = mu*rho; % Fluid dynamic viscosity [Pa s].
a = 0.0043; % Channel long side [m].
b = 0.00012; % Channel short side [m].
beta = b/a; % Channel aspect ratio.
NdivX = 101; % Resolusion - number of nodes in x direction (long side), odd number.
NdivY = 11; % Resolusion - number of nodes in y direction (short side), odd number.

disp('Program determines the fluid velocity profiles in a duct with the rectangular cross-section.');

% Velocity profile calculations (Spiga & Morino, 1994)
countx = 0;
V=zeros(NdivX,NdivY);
for x = linspace(0,1,NdivX)
    countx = countx + 1;
    county = 0;
    for y = linspace(0,beta,NdivY)
        county = county + 1;
        Vtemp = 0;
        for n = 1:2:500 % Number of infinite series elements to be sumarized.
            for m = 1:2:500 % Number of infinite series elements to be sumarized.
                Vtemp = Vtemp + (16*(beta^2)/(pi^4))*sin(n*pi*x)*sin(m*pi*y/beta)/(n*m*((beta^2)*n^2 + m^2));
            end
        end
        V(countx,county) = Vtemp;
    end
end

% Mean velocity calculations (Spiga & Morino, 1994)
Um=0;
for n = 1:2:300 % Number of infinite series elements to be sumarized.
  for m = 1:2:300 % Number of infinite series elements to be sumarized.
    Um = Um+(1/((n^2)*(m^2)*(n^2+(m^2)/(beta^2))));
  end
end
Um=Um*64/(pi^6);

% Normalized velocity magnitude [-] (Spiga & Morino, 1994)
V_norm = V/Um; 

% Purday’s model (1949)
clear x y n m;
V_P=zeros(NdivX,NdivY);

% Function for N and m Natarajan and Lakshmanan (1972)
if beta <= 1/3
    n = 2;
else
    n = 2 + 0.2*(beta - 1/3);
end
m=1.7+0.5*(beta^(-1.4));

countx = 0;
xx = abs(linspace(-a/2,a/2,NdivX));
yy = abs(linspace(-b/2,b/2,NdivY));
for x = xx
    countx = countx + 1;
    county = 0;
    for y = yy
        county = county + 1;
        V_norm_P(countx,county)= ((m+1)/m)*((n+1)/n)*(1-(y/(b/2))^n)*(1-(x/(a/2))^m);
    end
end

% Normalized geometric coordinates 
X_norm = linspace(-0.5,0.5,NdivX);
Y_norm = linspace(-0.5,0.5,NdivY);

% Figures
figure(1);
contourf(X_norm,Y_norm,V_norm');
xticks(-0.5:0.25:0.5);
yticks(-0.5:0.25:0.5);
colorbar;
title({'Normalized velocity distribution, Spiga & Morino (1994)'; ' '});
xlabel('x [-]');
ylabel('y [-]');

figure(2);
contourf(X_norm,Y_norm,V_norm_P');
xticks(-0.5:0.25:0.5);
yticks(-0.5:0.25:0.5);
colorbar;
title({'Normalized velocity distribution, Purday’s model (1949)'; ' '});
xlabel('x [-]');
ylabel('y [-]');

figure(3);
surf(Y_norm,X_norm,V_norm,V_norm,'FaceColor','interp');
xticks(-0.5:0.25:0.5);
yticks(-0.5:0.25:0.5);
colorbar;
title({'Normalized velocity profile: (a) Surface plot - Spiga & Morino model, (b) Red dots - Purday’s model'; ' '});
xlabel('y [-]');
ylabel('x [-]');
zlabel('u [-]');

PZ = reshape(V_norm_P,1,[]);
PX=[];
PY=[];
for i =1:length(Y_norm)
    PX = [PX,X_norm];
    for j =1:length(X_norm)
        PY = [PY,Y_norm(i)];
    end
end

hold on
plot3(PY,PX,PZ, 'ro','MarkerFaceColor','red');
hold off;

figure(5);
VhalfY = V_norm(:,round(NdivY/2))';
VhalfY_P = V_norm_P(:,round(NdivY/2))';
p5 = plot(X_norm,VhalfY,X_norm,VhalfY_P);
xticks(-0.5:0.125:0.5);
p5(1).Color = 'blue';
p5(2).Color = 'red';
title({'Normalized velocity profile at h = 1/2y'; ' '});
xlabel('x [-]');
ylabel('u [-]');
legend ('Spiga & Morino 1994','Purday’s model 1949','Location','best');

figure(6);
Vmean=[];
Vmean_P=[];
for i=1:NdivX
    Vmean = [Vmean,mean(V_norm(i,2:end))];
    Vmean_P = [Vmean_P,mean(V_norm_P(i,2:end))]; 
end;
p6 = plot(X_norm,Vmean,X_norm,Vmean_P);
xticks(-0.5:0.125:0.5);
p6(1).Color = 'blue';
p6(2).Color = 'red';
title({'y- mean normalized velocity'; ' '});
xlabel('x [-]');
ylabel('u [-]');
legend ('Spiga & Morino 1994','Purday’s model 1949','Location','best');

% Relative error, comparision between Purday’s model and Spiga & Morino model
V_err = (V_norm_P-V_norm)./V_norm_P*100;
V_err(isnan(V_err))=0; % NaN to 0 conversion
V_err(isinf(V_err))=0; % inf to 0 conversion
figure(7);
surf(Y_norm,X_norm,V_err,V_err,'FaceColor','interp');
xticks(-0.5:0.25:0.5);
yticks(-0.5:0.25:0.5);
colorbar;
title({'Two models comparision'; ' '});
xlabel('y [-]');
ylabel('x [-]');
zlabel('Divergence [%]');

%Results
Error_m = abs(mean(V_err,'all'));
Error_median = abs(median(V_err,'all'));
Error_SD = std(V_err,0,'all');
MeanErr_Vmean_y05 = mean((VhalfY-VhalfY_P)/VhalfY_P*100);
MeanErr_Vymean = mean((Vmean-Vmean_P)/Vmean_P*100);

info1 = ['Mean divergence between models = ',num2str(Error_m), ' [%]'];
info2 = ['Median divergence between models = ',num2str(Error_median), ' [%]'];
info3 = ['Divergence standard deviation ',num2str(Error_SD), ' [%]'];
info4 = ['Mean divergence of u/um between models for 1/2y ',num2str(MeanErr_Vmean_y05), ' [%]'];
info5 = ['Mean divergence of umy/um between models ',num2str(MeanErr_Vymean), ' [%]'];

disp(info1);
disp(info2);
disp(info3);
disp(info4);
disp(info5);
